/*
 nRF Doorbell Server
 Version: 1.0
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 15/11/2015
 Last Modified: 15/11/2015
 
 Listen for the doorbell TX and camera RX. 
 When the doorbell is pressed, play 2 tones on the speaker for 4 seconds, store the state so the camera RX can check in 
 and switch on, then reset the state after 25 seconds.
 
 Re-program ATtiny84 without clock divider by 8:
 avrdude -p ATtiny84 -c usbtiny -U lfuse:w:0xe2:m -U hfuse:w:0xdf:m -U efuse:w:0xff:m
 
 */
 
// ATtiny441/841 Pin map
//
//               +-\/-+
//         VCC  1|o   |14  GND
//         PB0  2|    |13  PA0 
//     LED PB1  3|    |12  PA1 
//   Reset PB3  4|    |11  PA2 Siren Mosfet
//      CE PB2  5|    |10  PA3 
//     CSN PA7  6|    |9   PA4 SCK
//      MI PA6  7|    |8   PA5 MO
//               +----+

#define F_CPU 8000000 // 8 MHz

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/sleep.h>
#include "setup.c"
#include "mirf.c"

volatile uint8_t doorbellState = SYSTEM_OFF;
volatile uint8_t speakerState = 0;

int main(void) {
	setup();
	
	// Test led and speaker
	TCCR0B = (1<<CS02);
	PORTB |= (1<<LED);
	_delay_ms(500);
	PORTB &= ~(1<<LED);
	TCCR0B = 0;
	PORTA &= ~(1<<speakerMosfetPin);
	
	NRF_RX_POWERUP; // Power up to receiver mode
	_delay_ms(3);
	
	while(1) {
		mirf_flush_rx_tx(); // Flush RX and TX FIFO
		mirf_CE_hi; // Start listening
		
		while (!(mirf_status() & (1<<RX_DR))) { // Wait until data is ready to be read
			_delay_us(50);
		}
		mirf_CE_lo; // Stop listening
		
		mirf_CSN_lo; // Pull down chip select
		spi_transfer(R_RX_PAYLOAD); // Send cmd to read rx payload
		uint8_t dataIn = spi_transfer(0x00); // Read payload
		mirf_CSN_hi; // Pull up chip select
		mirf_config_register(STATUS,(1<<RX_DR)); // Reset status register
		
		// If wake up packet received, store it
		if (dataIn == SYSTEM_ON || dataIn == SYSTEM_OFF) {
			doorbellState = dataIn;
		}
		else if (dataIn == SYSTEM_REQUEST_STATE) { // Request of state made by camera RX
			mirf_transmit_data(doorbellState);
			
			NRF_RX_POWERUP; // Power up to receiver mode
			_delay_ms(3);
		}
		
		// Sound speaker and turn on monitor
		if (dataIn == SYSTEM_ON) {
			speakerState = 0;
			
			// Turn on speaker
			TCNT0 = 0;
			TCCR0B = (1<<CS02);
			OCR0A = 30;
			
			// Turn on timer 1
			TCNT1 = 0;
			TCCR1B = (1<<CS11) | (1<<CS10);
		}
		
		// Light LED when a packet is received
		PORTB |= (1<<LED);
		_delay_ms(50);
		PORTB &= ~(1<<LED);
	}
}

// Timer0 CTC, on compare switch speaker on/off
ISR(TIM0_COMPA_vect) {
	if (OCR0A != 0) {
		PORTA ^= (1<<speakerMosfetPin);
	}
	else {
		PORTA &= ~(1<<speakerMosfetPin);
	}
}

// Timer1 overflow, every ~0.5 seconds
ISR(TIM1_OVF_vect) {
	if (speakerState == 2) { // Change the speaker tone
		OCR0A = 40;
	}
	else if (speakerState == 4) { // Turn off speaker
		OCR0A = 0;
		TCCR0B = 0;
		PORTA &= ~(1<<speakerMosfetPin);
	}
	else if (speakerState >= 50) { // Reset doorbell state after ~25 seconds
		doorbellState = SYSTEM_OFF;
		TCCR1B = 0;
	}
	
	speakerState++;
}